@@ -1,10 +1,10 @@
 package Net::DNS::RR::RRSIG;
 
 #
-# $Id: RRSIG.pm 1293 2015-01-07 07:42:26Z willem $
+# $Id: RRSIG.pm 1302 2015-01-23 11:07:41Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1293 $)[1];
+$VERSION = (qw$LastChangedRevision: 1302 $)[1];
 
 
 use strict;
@@ -28,22 +28,15 @@ use constant UTIL => eval { require Scalar::Util; } || 0;
 
 use Net::DNS::Parameters;
 
-eval { require Net::DNS::SEC::Private };
-
 my $debug = 0;
 
-BEGIN {
-	eval { require Crypt::OpenSSL::RSA };
-	eval { require Crypt::OpenSSL::Bignum };
-	eval { require Digest::SHA };
+use constant DNSSEC => eval { require Net::DNS::SEC::Private } || 0;
 
-	use constant DSA => eval { require Crypt::OpenSSL::DSA } || 0;
-	use constant EC	 => eval { require Crypt::OpenSSL::EC }	 || 0;
-	use constant ECDSA => EC && eval { require Crypt::OpenSSL::ECDSA }   || 0;
-	use constant GOST  => EC && eval { require Digest::GOST::CryptoPro } || 0;
+use constant DSA => eval { require Net::DNS::SEC::DSA; 'Net::DNS::SEC::DSA' } || 0;
+use constant RSA => eval { require Net::DNS::SEC::RSA; 'Net::DNS::SEC::RSA' } || 0;
 
-	eval { require Crypt::OpenSSL::Random } if GOST;
-}
+use constant ECDSA => eval { require Net::DNS::SEC::ECDSA;   'Net::DNS::SEC::ECDSA' }	|| 0;
+use constant GOST  => eval { require Net::DNS::SEC::ECCGOST; 'Net::DNS::SEC::ECCGOST' } || 0;
 
 
 sub decode_rdata {			## decode rdata from wire-format octet string
@@ -88,27 +81,6 @@ sub parse_rdata {			## populate RR from rdata in argument list
 }
 
 
-my %RSA = (
-	'1'  => 'use_md5_hash',
-	'5'  => 'use_sha1_hash',
-	'7'  => 'use_sha1_hash',
-	'8'  => 'use_sha256_hash',
-	'10' => 'use_sha512_hash',
-	);
-
-my %DSA = (
-	'3' => ['Digest::SHA'],
-	'6' => ['Digest::SHA'],
-	);
-
-my %GOST = ( '12' => ['Digest::GOST::CryptoPro'] );
-
-my %ECDSA = (
-	'13' => ['Digest::SHA', 256],
-	'14' => ['Digest::SHA', 384],
-	);
-
-
 #
 # source: http://www.iana.org/assignments/dns-sec-alg-numbers
 #
@@ -156,6 +128,20 @@ my %ECDSA = (
 }
 
 
+my %SEC = (
+	1  => RSA,
+	3  => DSA,
+	5  => RSA,
+	6  => DSA,
+	7  => RSA,
+	8  => RSA,
+	10 => RSA,
+	12 => GOST,
+	13 => ECDSA,
+	14 => ECDSA,
+	);
+
+
 sub typecovered {
 	my $self = shift;
 	$self->{typecovered} = typebyname(shift) if scalar @_;
@@ -247,8 +233,10 @@ sub sigbin {
 sub create {
 	my ( $class, $datarrset, $priv_key, %args ) = @_;
 
+	croak 'Net::DNS::SEC support not available' unless DNSSEC;
+
 	my $private = ref($priv_key) ? $priv_key : Net::DNS::SEC::Private->new($priv_key);
-	croak 'Unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
+	croak 'unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
 
 	croak '$datarrset argument is not a reference to an array' unless ref($datarrset) =~ /ARRAY/;
 
@@ -591,13 +579,10 @@ sub _CreateSig {
 
 	my $algorithm = $self->algorithm;
 
-	return $self->_CreateRSA(@_) if $RSA{$algorithm};
-	return $self->_CreateDSA(@_) if DSA && $DSA{$algorithm};
-
-	return $self->_CreateECDSA(@_) if ECDSA && $ECDSA{$algorithm};
-	return $self->_CreateGOST(@_)  if GOST	&& $GOST{$algorithm};
-
-	croak "Signature generation not supported for algorithm $algorithm";
+	eval {
+		my $class = $SEC{$algorithm} || die "algorithm $algorithm not supported";
+		$self->sigbin( $class->generate(@_) );
+	} || croak 'signature generation failed', $@ ? "\n\t$@" : '';
 }
 
 
@@ -606,400 +591,21 @@ sub _VerifySig {
 
 	my $algorithm = $self->algorithm;
 
-	return $self->_VerifyRSA(@_) if $RSA{$algorithm};
-	return $self->_VerifyDSA(@_) if DSA && $DSA{$algorithm};
-
-	return $self->_VerifyECDSA(@_) if ECDSA && $ECDSA{$algorithm};
-	return $self->_VerifyGOST(@_)  if GOST	&& $GOST{$algorithm};
-
-	$self->{vrfyerrstr} = "Verification not supported for algorithm $algorithm";
-	return 0;
-}
-
-
-sub _CreateRSA {
-	my ( $self, $sigdata, $private ) = @_;
-
-	my $hash = $RSA{$private->algorithm} || croak 'private key not RSA';
-
-	eval {
-		my @param = map Crypt::OpenSSL::Bignum->new_from_bin( decode_base64( $private->$_ ) ),
-				qw(Modulus PublicExponent PrivateExponent
-				Prime1 Prime2 Exponent1 Exponent2 Coefficient);
-		my $rsa = Crypt::OpenSSL::RSA->new_key_from_parameters(@param);
-
-		$rsa->use_pkcs1_oaep_padding;
-		$rsa->$hash;
-		$self->sigbin( $rsa->sign($sigdata) );
-	} || carp "RSA signature generation failed\n\t$@";
-}
-
-
-sub _VerifyRSA {
-	my ( $self, $sigdata, $keyrr ) = @_;
-
-	# Implementation using Crypt::OpenSSL::RSA
-
-	print "\n_VerifyRSA:\n", $self->string, "\nusing key:\n", $keyrr->string, "\n" if $debug;
-
-	eval {
-		my $keybin = $keyrr->keybin;			# public key
-		my ( $exponent, $modulus );			# RFC 2537, section 2
-		if ( my $explength = unpack( 'C', $keybin ) ) {
-			( $exponent, $modulus ) = unpack( "x a$explength a*", $keybin );
-		} else {
-			$explength = unpack( 'xn', $keybin );
-			( $exponent, $modulus ) = unpack( "x3 a$explength a*", $keybin );
-		}
-
-		my $bn_modulus	= Crypt::OpenSSL::Bignum->new_from_bin($modulus);
-		my $bn_exponent = Crypt::OpenSSL::Bignum->new_from_bin($exponent);
-
-		my $rsa = Crypt::OpenSSL::RSA->new_key_from_parameters( $bn_modulus, $bn_exponent );
-
-		my $hash = $RSA{$self->algorithm};
-		$rsa->use_pkcs1_oaep_padding;
-		$rsa->$hash;
-
-		$rsa->verify( $sigdata, $self->sigbin );
-	} || do {
-		$self->{vrfyerrstr} = "RSA Verification failed";
-		$self->{vrfyerrstr} = "RSA verification error: $@" if $@;
-		print "\n", $self->{vrfyerrstr}, "\n" if $debug;
-		return 0;
-	};
-
-	print "\nRSA verification successful\n" if $debug;
-	return 1;
-}
-
-
-sub _CreateDSA {
-	my ( $self, $sigdata, $private ) = @_;
-
-	eval {
-		my $algorithm = $private->algorithm;		# digest sigdata
-		my ( $object, @param ) = @{$DSA{$algorithm}};
-		croak 'private key not DSA' unless $object;
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-
-		my $dsa = Crypt::OpenSSL::DSA->new();		# private key
-		$dsa->set_p( decode_base64 $private->prime );
-		$dsa->set_q( decode_base64 $private->subprime );
-		$dsa->set_g( decode_base64 $private->base );
-		$dsa->set_priv_key( decode_base64 $private->private_value );
-
-		my $dsasig = $dsa->do_sign( $hash->digest );
-
-		# See RFC 2535 for the content of the SIG
-		my $T = ( length( $dsa->get_g ) - 64 ) / 8;
-		my $R = $dsasig->get_r;
-		my $S = $dsasig->get_s;
-
-		# both the R and S parameters need to be 20 octets:
-		my $Rpad = 20 - length($R);
-		my $Spad = 20 - length($S);
-		$self->sigbin( pack "C x$Rpad a* x$Spad a*", $T, $R, $S );
-	} || carp "DSA signature generation failed\n\t$@";
-}
-
-
-sub _VerifyDSA {
-	my ( $self, $sigdata, $keyrr ) = @_;
-
-	print "\n_VerifyDSA:\n", $self->string, "\nusing key:\n", $keyrr->string, "\n" if $debug;
-
 	my $retval = eval {
-		my $algorithm = $self->algorithm;		# digest sigdata
-		my ( $object, @param ) = @{$DSA{$algorithm}};
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-
-		my $keybin = $keyrr->keybin;			# public key
-		my $numlen = 64 + 8 * unpack( 'C', $keybin );
-		my ( $q, $p, $g, $k ) = unpack "x a20 a$numlen a$numlen a$numlen", $keybin;
-
-		my $dsa = Crypt::OpenSSL::DSA->new();
-		$dsa->set_q($q);
-		$dsa->set_g($g);
-		$dsa->set_p($p);
-		$dsa->set_pub_key($k);
-
-		my $dsasig = Crypt::OpenSSL::DSA::Signature->new();
-		my ( $r, $s ) = unpack 'x a20 a20', $self->sigbin;
-		$dsasig->set_r($r);
-		$dsasig->set_s($s);
-
-		$dsa->do_verify( $hash->digest, $dsasig );
+		my $class = $SEC{$algorithm} || die "algorithm $algorithm not supported";
+		$class->verify( @_, $self->sigbin );
 	} || do {
-		$self->{vrfyerrstr} = "DSA verification failed";
-		$self->{vrfyerrstr} = "DSA verification error: $@" if $@;
+		$self->{vrfyerrstr} = 'signature verification failed';
+		$self->{vrfyerrstr} .= "\n\t$@" if $@;
 		print "\n", $self->{vrfyerrstr}, "\n" if $debug;
 		return 0;
 	};
 
-	croak 'Error in DSA_do_verify' unless $retval == 1;	# fix for DSA < 0.14
-	print "\nDSA verification successful\n" if $debug;
+	croak "unknown error in algorithm $algorithm verify" unless $retval == 1;
+	print "\nalgorithm $algorithm verification successful\n" if $debug;
 	return 1;
 }
 
-
-BEGIN {
-	use vars qw(%ECcurve);
-
-	my %NIST_P256 = (		## FIPS 186-4, D.1.2.3
-		p => 'ffffffff00000001000000000000000000000000ffffffffffffffffffffffff',
-		a => 'ffffffff00000001000000000000000000000000fffffffffffffffffffffffc',    # -3 mod p
-		b => '5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b',
-		x => '6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296',
-		y => '4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5',
-		n => 'ffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551',
-		);
-
-	my %NIST_P384 = (		## FIPS 186-4, D.1.2.4
-		p => 'fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff',
-		a => 'fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000fffffffc',
-		b => 'b3312fa7e23ee7e4988e056be3f82d19181d9c6efe8141120314088f5013875ac656398d8a2ed19d2a85c8edd3ec2aef',
-		x => 'aa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a385502f25dbf55296c3a545e3872760ab7',
-		y => '3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c00a60b1ce1d7e819d7a431d7c90ea0e5f',
-		n => 'ffffffffffffffffffffffffffffffffffffffffffffffffc7634d81f4372ddf581a0db248b0a77aecec196accc52973',
-		);
-
-	my %GOST_R_34_10_2001_CryptoPro_A = (			# RFC4357
-		a => 'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD94',    # -3 mod p
-		b => '00A6',					# 166
-		p => 'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD97',
-		n => 'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF6C611070995AD10045841B09B761B893',    # q
-		x => '01',
-		y => '8D91E471E0989CDA27DF505A453F2B7635294F2DDF23E3B122ACC99C9E9F1E14'
-		);
-
-
-	my $_curve = sub {
-		return unless EC;
-
-		my %param = @_;
-
-		my $p = Crypt::OpenSSL::Bignum->new_from_hex( $param{p} );
-		my $a = Crypt::OpenSSL::Bignum->new_from_hex( $param{a} );
-		my $b = Crypt::OpenSSL::Bignum->new_from_hex( $param{b} );
-		my $x = Crypt::OpenSSL::Bignum->new_from_hex( $param{x} );
-		my $y = Crypt::OpenSSL::Bignum->new_from_hex( $param{y} );
-		my $n = Crypt::OpenSSL::Bignum->new_from_hex( $param{n} );
-		my $h = Crypt::OpenSSL::Bignum->one;
-
-		my $ctx	   = Crypt::OpenSSL::Bignum::CTX->new();
-		my $method = Crypt::OpenSSL::EC::EC_GFp_mont_method();
-		my $group  = Crypt::OpenSSL::EC::EC_GROUP::new($method);
-		$group->set_curve_GFp( $p, $a, $b, $ctx );	# y^2 = x^3 + a*x + b  mod p
-
-		my $G = Crypt::OpenSSL::EC::EC_POINT::new($group);
-		Crypt::OpenSSL::EC::EC_POINT::set_affine_coordinates_GFp( $group, $G, $x, $y, $ctx );
-		$group->set_generator( $G, $n, $h );
-		die 'bad curve' unless Crypt::OpenSSL::EC::EC_GROUP::check( $group, $ctx );
-		return $group;
-	};
-
-	$ECcurve{12} = &$_curve(%GOST_R_34_10_2001_CryptoPro_A);
-	$ECcurve{13} = &$_curve(%NIST_P256);
-	$ECcurve{14} = &$_curve(%NIST_P384);
-}
-
-
-sub _CreateECDSA {
-	my ( $self, $sigdata, $private ) = @_;
-
-	eval {
-		my $algorithm = $self->algorithm;
-		my $group     = $ECcurve{$algorithm}->dup();	# precalculated curve
-
-		my ( $object, @param ) = @{$ECDSA{$algorithm}}; # digest sigdata
-		croak 'private key not ECDSA' unless $object;
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-		my $digest = $hash->digest;
-
-		my $keybin = decode_base64( $private->PrivateKey );
-		my $bignum = Crypt::OpenSSL::Bignum->new_from_bin($keybin);
-
-		my $eckey = Crypt::OpenSSL::EC::EC_KEY::new();
-		$eckey->set_group($group)	 || die;
-		$eckey->set_private_key($bignum) || die;
-
-		my $ecsig = Crypt::OpenSSL::ECDSA::ECDSA_do_sign( $digest, $eckey );
-		my ( $r, $s ) = ( $ecsig->get_r, $ecsig->get_s );
-
-		# both the r and s parameters need to be zero padded:
-		my $size = length $digest;
-		my $Rpad = $size - length $r;
-		my $Spad = $size - length $s;
-		$self->sigbin( pack "x$Rpad a* x$Spad a*", $r, $s );
-	} || carp "ECDSA signature generation failed\n\t$@";
-}
-
-
-sub _VerifyECDSA {
-	my ( $self, $sigdata, $keyrr ) = @_;
-
-	# Implementation using Crypt::OpenSSL::ECDSA
-
-	print "\n_VerifyECDSA:\n", $self->string, "\nusing key:\n", $keyrr->string, "\n" if $debug;
-
-	my $retval = eval {
-		my $algorithm = $self->algorithm;
-		my $group     = $ECcurve{$algorithm}->dup();	# precalculated curve
-
-		my ( $object, @param ) = @{$ECDSA{$algorithm}}; # digest sigdata
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-		my $digest = $hash->digest;
-
-		my $keybin = $keyrr->keybin;			# public key
-		my $keylen = length($keybin) >> 1;
-		my ( $x, $y ) = map Crypt::OpenSSL::Bignum->new_from_bin($_), unpack "a$keylen a*", $keybin;
-		my $key = Crypt::OpenSSL::EC::EC_POINT::new($group);
-		my $ctx = Crypt::OpenSSL::Bignum::CTX->new();
-		Crypt::OpenSSL::EC::EC_POINT::set_affine_coordinates_GFp( $group, $key, $x, $y, $ctx );
-
-		my $eckey = Crypt::OpenSSL::EC::EC_KEY::new();
-		$eckey->set_group($group)    || die;
-		$eckey->set_public_key($key) || die;
-
-		my $sigbin = $self->sigbin;			# signature
-		my $siglen = length($sigbin) >> 1;
-		my ( $r, $s ) = unpack( "a$siglen a*", $sigbin );
-
-		my $dsasig = Crypt::OpenSSL::ECDSA::ECDSA_SIG->new();
-		$dsasig->set_r($r);
-		$dsasig->set_s($s);
-
-		Crypt::OpenSSL::ECDSA::ECDSA_do_verify( $digest, $dsasig, $eckey );
-	} || do {
-		$self->{vrfyerrstr} = "ECDSA verification failed";
-		$self->{vrfyerrstr} = "ECDSA verification error: $@" if $@;
-		print "\n", $self->{vrfyerrstr}, "\n" if $debug;
-		return 0;
-	};
-
-	croak 'Error in ECDSA_do_verify' unless $retval == 1;
-	print "\nECDSA verification successful\n" if $debug;
-	return 1;
-}
-
-
-sub _CreateGOST {
-	my ( $self, $sigdata, $private ) = @_;
-
-	eval {
-		my $algorithm = $self->algorithm;
-		my $group     = $ECcurve{$algorithm}->dup();	# precalculated curve
-
-		my $ctx = Crypt::OpenSSL::Bignum::CTX->new();
-		my $key = decode_base64( $private->PrivateKey );
-		my $d	= Crypt::OpenSSL::Bignum->new_from_bin($key);
-		my $P	= $group->get0_generator();
-
-		my ( $object, @param ) = @{$GOST{$algorithm}};	# step 1	(nomenclature per RFC5832)
-		croak 'private key not GOST' unless $object;
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-		my $H = reverse $hash->digest;
-
-		my $q = Crypt::OpenSSL::Bignum->zero;		# step 2
-		$group->get_order( $q, $ctx );
-		my $alpha = Crypt::OpenSSL::Bignum->new_from_bin($H);
-		my $e = $alpha->mod( $q, $ctx );    # Note: alpha can exceed but is never longer than q
-		$e = Crypt::OpenSSL::Bignum->one if $e->is_zero;
-
-		my $zeta;
-		{						# step 3
-			$hash->add($key);
-			$hash->add( Crypt::OpenSSL::Random::random_bytes(10) );
-			$hash->add( $hash->digest );
-			my $m = Crypt::OpenSSL::Bignum->new_from_bin( $hash->digest );
-			my $k = $m->mod( $q, $ctx );
-			redo if $k->is_zero;
-
-			my $x = Crypt::OpenSSL::Bignum->zero;	# step 4
-			my $y = Crypt::OpenSSL::Bignum->zero;
-			my $C = Crypt::OpenSSL::EC::EC_POINT::new($group);
-			Crypt::OpenSSL::EC::EC_POINT::mul( $group, $C, $q, $P, $k, $ctx );
-			Crypt::OpenSSL::EC::EC_POINT::get_affine_coordinates_GFp( $group, $C, $x, $y, $ctx );
-			my $r = $x->mod( $q, $ctx );
-			redo if $r->is_zero;
-
-			my $v = $r->mul( $d, $ctx );		# step 5
-			my $w = $k->mul( $e, $ctx );
-			my $s = $v->add($w)->mod( $q, $ctx );
-			redo if $s->is_zero;
-
-			my $size = length $H;			# step 6
-			$zeta = pack "a$size a$size", reverse( $s->to_bin ), reverse( $r->to_bin );
-		}
-
-		$self->sigbin($zeta);
-	} || croak "GOST signature generation failed\n\t$@";
-}
-
-
-sub _VerifyGOST {
-	my ( $self, $sigdata, $keyrr ) = @_;
-
-	# Implementation (ab)using Crypt::OpenSSL::ECDSA
-
-	print "\n_VerifyGOST:\n", $self->string, "\nusing key:\n", $keyrr->string, "\n" if $debug;
-
-	my $retval = eval {
-		my $algorithm = $self->algorithm;
-		my $group     = $ECcurve{$algorithm}->dup();	# precalculated curve
-
-		my $zeta = $self->sigbin;			# step 1	(nomenclature per RFC5832)
-		my $size = length($zeta) >> 1;
-		my ( $s, $r ) = unpack( "a$size a*", $zeta );
-
-		my ( $object, @param ) = @{$GOST{$algorithm}};	# step 2
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-		my $H = reverse $hash->digest;
-
-		my $ctx = Crypt::OpenSSL::Bignum::CTX->new();	# step 3
-		my $q	= Crypt::OpenSSL::Bignum->zero;
-		$group->get_order( $q, $ctx );
-		my $alpha = Crypt::OpenSSL::Bignum->new_from_bin($H);
-		my $e = $alpha->mod( $q, $ctx );    # Note: alpha can exceed but is never longer than q
-		$e = Crypt::OpenSSL::Bignum->one if $e->is_zero;
-
-		my $keybin = reverse $keyrr->keybin;		# public key
-		my $keylen = length($keybin) >> 1;
-		my ( $y, $x ) = map Crypt::OpenSSL::Bignum->new_from_bin($_), unpack "a$keylen a*", $keybin;
-		my $Q = Crypt::OpenSSL::EC::EC_POINT::new($group);
-		Crypt::OpenSSL::EC::EC_POINT::set_affine_coordinates_GFp( $group, $Q, $x, $y, $ctx );
-
-		my $eckey = Crypt::OpenSSL::EC::EC_KEY::new();
-		$eckey->set_group($group)  || die;
-		$eckey->set_public_key($Q) || die;
-
-		# algebraic transformation of ECC-GOST into equivalent ECDSA problem
-		my $dsasig = Crypt::OpenSSL::ECDSA::ECDSA_SIG->new();
-		$dsasig->set_r($r);
-		$dsasig->set_s( $q->sub($e)->to_bin );
-
-		my $m = $q->sub( Crypt::OpenSSL::Bignum->new_from_bin($s) )->mod( $q, $ctx );
-		Crypt::OpenSSL::ECDSA::ECDSA_do_verify( $m->to_bin, $dsasig, $eckey );
-	} || do {
-		$self->{vrfyerrstr} = "EC-GOST verification failed";
-		$self->{vrfyerrstr} = "EC-GOST verification error: $@" if $@;
-		print "\n", $self->{vrfyerrstr}, "\n" if $debug;
-		return 0;
-	};
-
-	croak 'Error in ECDSA_do_verify' unless $retval == 1;
-	print "\nECC-GOST verification successful\n" if $debug;
-	return 1;
-}
-
-
 1;
 __END__
 
@@ -1012,8 +618,8 @@ __END__
 				keytag signame signature');
 
     $sigrr = create Net::DNS::RR::RRSIG( \@rrset, $keypath,
-					sigex => 20140731010101
-					sigin => 20140701010101
+					sigex => 20151231010101
+					sigin => 20151201010101
 					);
 
     $sigrr->verify( \@rrset, $keyrr ) || die $sigrr->vrfyerrstr;
@@ -1131,8 +737,8 @@ Create a signature over a RR set.
     $sigrr = create Net::DNS::RR::RRSIG( \@datarrset, $keypath );
 
     $sigrr = create Net::DNS::RR::RRSIG( \@datarrset, $keypath,
-					sigex => 20140731010101
-					sigin => 20140701010101
+					sigex => 20151231010101
+					sigin => 20151201010101
 					);
     $sigrr->print;
 
@@ -1158,8 +764,8 @@ containing the private key as generated by dnssec-keygen.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigex  => 20140731010101,	# signature expiration
-	sigin  => 20140701010101,	# signature inception
+	sigex  => 20151231010101,	# signature expiration
+	sigin  => 20151201010101,	# signature inception
 	sigval => 30,			# validity window (days)
 	ttl    => 3600			# TTL
 
