@@ -14,10 +14,10 @@ sub UNITCHECK {				## restore %SIG after compilation
 package Net::DNS::RR::SIG;
 
 #
-# $Id: SIG.pm 1291 2015-01-06 07:44:34Z willem $
+# $Id: SIG.pm 1302 2015-01-23 11:07:41Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1291 $)[1];
+$VERSION = (qw$LastChangedRevision: 1302 $)[1];
 
 
 use strict;
@@ -41,17 +41,12 @@ use constant UTIL => eval { require Scalar::Util; } || 0;
 
 use Net::DNS::Parameters;
 
-eval { require Net::DNS::SEC::Private };
-
 my $debug = 0;
 
-BEGIN {
-	eval { require Crypt::OpenSSL::RSA };
-	eval { require Crypt::OpenSSL::Bignum };
-	eval { require Digest::SHA };
+use constant DNSSEC => eval { require Net::DNS::SEC::Private } || 0;
 
-	use constant DSA => eval { require Crypt::OpenSSL::DSA } || 0;
-}
+use constant DSA => eval { require Net::DNS::SEC::DSA; 'Net::DNS::SEC::DSA' } || 0;
+use constant RSA => eval { require Net::DNS::SEC::RSA; 'Net::DNS::SEC::RSA' } || 0;
 
 
 sub decode_rdata {			## decode rdata from wire-format octet string
@@ -122,14 +117,6 @@ sub defaults() {			## specify RR attribute default values
 }
 
 
-my %RSA = (
-	'1' => 'use_md5_hash',
-	'5' => 'use_sha1_hash'
-	);
-
-my %DSA = ( '3' => ['Digest::SHA'] );
-
-
 #
 # source: http://www.iana.org/assignments/dns-sec-alg-numbers
 #
@@ -177,6 +164,15 @@ my %DSA = ( '3' => ['Digest::SHA'] );
 }
 
 
+my %SEC = (
+	1 => RSA,
+	3 => DSA,
+	5 => RSA,
+	6 => DSA,
+	7 => RSA,
+	);
+
+
 sub typecovered {
 	my $self = shift;
 	$self->{typecovered} = typebyname(shift) if scalar @_;
@@ -266,6 +262,8 @@ sub sigbin {
 sub create {
 	my ( $class, $data, $priv_key, %args ) = @_;
 
+	croak 'Net::DNS::SEC support not available' unless DNSSEC;
+
 	my $private = ref($priv_key) ? $priv_key : Net::DNS::SEC::Private->new($priv_key);
 	croak 'Unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
 
@@ -490,10 +488,10 @@ sub _CreateSig {
 
 	my $algorithm = $self->algorithm;
 
-	return $self->_CreateRSA(@_) if $RSA{$algorithm};
-	return $self->_CreateDSA(@_) if DSA && $DSA{$algorithm};
-
-	croak "Signature generation not supported for algorithm $algorithm";
+	eval {
+		my $class = $SEC{$algorithm} || die "algorithm $algorithm not supported";
+		$self->sigbin( $class->generate(@_) );
+	} || croak 'signature generation failed', $@ ? "\n\t$@" : '';
 }
 
 
@@ -502,138 +500,18 @@ sub _VerifySig {
 
 	my $algorithm = $self->algorithm;
 
-	return $self->_VerifyRSA(@_) if $RSA{$algorithm};
-	return $self->_VerifyDSA(@_) if DSA && $DSA{$algorithm};
-
-	$self->{vrfyerrstr} = "Verification not supported for algorithm $algorithm";
-	return 0;
-}
-
-
-sub _CreateRSA {
-	my ( $self, $sigdata, $private ) = @_;
-
-	my $hash = $RSA{$private->algorithm} || croak 'private key not RSA';
-
-	eval {
-		my @param = map Crypt::OpenSSL::Bignum->new_from_bin( decode_base64( $private->$_ ) ),
-				qw(Modulus PublicExponent PrivateExponent
-				Prime1 Prime2 Exponent1 Exponent2 Coefficient);
-		my $rsa = Crypt::OpenSSL::RSA->new_key_from_parameters(@param);
-
-		$rsa->use_pkcs1_oaep_padding;
-		$rsa->$hash;
-		$self->sigbin( $rsa->sign($sigdata) );
-	} || carp "RSA signature generation failed\n\t$@";
-}
-
-
-sub _VerifyRSA {
-	my ( $self, $sigdata, $keyrr ) = @_;
-
-	# Implementation using Crypt::OpenSSL::RSA
-
-	print "\n_VerifyRSA:\n", $self->string, "\nusing key:\n", $keyrr->string, "\n" if $debug;
-
-	eval {
-		my $keybin = $keyrr->keybin;			# public key
-		my ( $exponent, $modulus );			# RFC 2537, section 2
-		if ( my $explength = unpack( 'C', $keybin ) ) {
-			( $exponent, $modulus ) = unpack( "x a$explength a*", $keybin );
-		} else {
-			$explength = unpack( 'xn', $keybin );
-			( $exponent, $modulus ) = unpack( "x3 a$explength a*", $keybin );
-		}
-
-		my $bn_modulus	= Crypt::OpenSSL::Bignum->new_from_bin($modulus);
-		my $bn_exponent = Crypt::OpenSSL::Bignum->new_from_bin($exponent);
-
-		my $rsa = Crypt::OpenSSL::RSA->new_key_from_parameters( $bn_modulus, $bn_exponent );
-
-		my $hash = $RSA{$self->algorithm};
-		$rsa->use_pkcs1_oaep_padding;
-		$rsa->$hash;
-
-		$rsa->verify( $sigdata, $self->sigbin );
-	} || do {
-		$self->{vrfyerrstr} = "RSA Verification failed";
-		$self->{vrfyerrstr} = "RSA verification error: $@" if $@;
-		print "\n", $self->{vrfyerrstr}, "\n" if $debug;
-		return 0;
-	};
-
-	print "\nRSA verification successful\n" if $debug;
-	return 1;
-}
-
-
-sub _CreateDSA {
-	my ( $self, $sigdata, $private ) = @_;
-
-	eval {
-		my $algorithm = $private->algorithm;		# digest sigdata
-		my ( $object, @param ) = @{$DSA{$algorithm}};
-		croak 'private key not DSA' unless $object;
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-
-		my $dsa = Crypt::OpenSSL::DSA->new();		# private key
-		$dsa->set_p( decode_base64 $private->prime );
-		$dsa->set_q( decode_base64 $private->subprime );
-		$dsa->set_g( decode_base64 $private->base );
-		$dsa->set_priv_key( decode_base64 $private->private_value );
-
-		my $dsasig = $dsa->do_sign( $hash->digest );
-
-		# See RFC 2535 for the content of the SIG
-		my $T = ( length( $dsa->get_g ) - 64 ) / 8;
-		my $R = $dsasig->get_r;
-		my $S = $dsasig->get_s;
-
-		# both the R and S parameters need to be 20 octets:
-		my $Rpad = 20 - length($R);
-		my $Spad = 20 - length($S);
-		$self->sigbin( pack "C x$Rpad a* x$Spad a*", $T, $R, $S );
-	} || carp "DSA signature generation failed\n\t$@";
-}
-
-
-sub _VerifyDSA {
-	my ( $self, $sigdata, $keyrr ) = @_;
-
-	print "\n_VerifyDSA:\n", $self->string, "\nusing key:\n", $keyrr->string, "\n" if $debug;
-
 	my $retval = eval {
-		my $algorithm = $self->algorithm;		# digest sigdata
-		my ( $object, @param ) = @{$DSA{$algorithm}};
-		my $hash = $object->new(@param);
-		$hash->add($sigdata);
-
-		my $keybin = $keyrr->keybin;			# public key
-		my $numlen = 64 + 8 * unpack( 'C', $keybin );
-		my ( $q, $p, $g, $k ) = unpack "x a20 a$numlen a$numlen a$numlen", $keybin;
-
-		my $dsa = Crypt::OpenSSL::DSA->new();
-		$dsa->set_q($q);
-		$dsa->set_g($g);
-		$dsa->set_p($p);
-		$dsa->set_pub_key($k);
-
-		my $dsasig = Crypt::OpenSSL::DSA::Signature->new();
-		my ( $r, $s ) = unpack 'x a20 a20', $self->sigbin;
-		$dsasig->set_r($r);
-		$dsasig->set_s($s);
-
-		$dsa->do_verify( $hash->digest, $dsasig );
+		my $class = $SEC{$algorithm} || die "algorithm $algorithm not supported";
+		$class->verify( @_, $self->sigbin );
 	} || do {
-		$self->{vrfyerrstr} = "DSA verification failed";
-		$self->{vrfyerrstr} = "DSA verification error: $@" if $@;
+		$self->{vrfyerrstr} = 'signature verification failed';
+		$self->{vrfyerrstr} .= "\n\t$@" if $@;
 		print "\n", $self->{vrfyerrstr}, "\n" if $debug;
 		return 0;
 	};
 
-	croak 'Error in DSA_do_verify' unless $retval == 1;	# fix for DSA < 0.14
-	print "\nDSA verification successful\n" if $debug;
+	croak "unknown error in algorithm $algorithm verify" unless $retval == 1;
+	print "\nalgorithm $algorithm verification successful\n" if $debug;
 	return 1;
 }
 
@@ -770,8 +648,8 @@ that comes with the ISC BIND distribution.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigin  => 20140701010101,	# signature inception
-	sigex  => 20140701011101,	# signature expiration
+	sigin  => 20151201010101,	# signature inception
+	sigex  => 20151201011101,	# signature expiration
 	sigval => 10,			# validity window (minutes)
 
 The sigin and sigex values may be specified as Perl time values or as
